<?php

namespace Drupal\vat_number\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\vat_number\Controller\VatNumberController;

/**
 * Plugin implementation of the 'vat_widget' widget.
 *
 * @FieldWidget(
 *   id = "vat_widget",
 *   module = "vat_number",
 *   label = @Translation("VAT Number"),
 *   field_types = {
 *     "vat_number"
 *   }
 * )
 */
class VatNumberWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'validate_vies' => TRUE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['validate_vies'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Validate against VIES database'),
      '#description' => $this->t('If checked, the entered VAT number will be validated against the European VIES VAT database.'),
      '#default_value' => $this->getSetting('validate_vies'),
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $validate_vies = $this->getSetting('validate_vies');
    $value = $validate_vies ? $this->t('Yes') : $this->t('No');
    $summary[] = $this->t('VIES database validation: @value', ['@value' => $value]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element += [
      '#type' => 'textfield',
      '#default_value' => isset($items[$delta]->getValue()['value']) ? $items[$delta]->getValue()['value'] : NULL,
      '#element_validate' => [
        [$this, 'validate'],
      ],
    ];

    return ['value' => $element];
  }

  /**
   * Validate the fields and check if the vat number is valid.
   */
  public function validate($element, FormStateInterface $form_state) {
    $value = $element['#value'];
    if (!empty($value)) {

      // Include the VAT Controller.
      $vatController = new VatNumberController($value);

      // Validate VAT number.
      $valid = $vatController->check($this->getSetting('validate_vies'));
      if (!$valid['status']) {
        $form_state->setError($element, $valid['message']);
      }
    }
  }

}
